/*
 *      dcctabitem.cpp
 *
 *      Copyright 2010 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <string>
#include "dcctabitem.h"
#include "dxutils.h"
#include "tetristabitem.h"
#include "icons.h"
#include "config.h"
#include "i18n.h"
#include "engine/ircengine.h"
#include "dxirc.h"

#define preferences dxirc::instance()->getPreferences()

FXDEFMAP(DccTabItem) DccTabItemMap[] = {
    FXMAPFUNC(SEL_COMMAND,          DccTabItem_COMMANDLINE,     DccTabItem::onCommandline),
    FXMAPFUNC(SEL_KEYPRESS,         DccTabItem_COMMANDLINE,     DccTabItem::onKeyPress),
    FXMAPFUNC(SEL_TIMEOUT,          DccTabItem_PTIME,           DccTabItem::onPipeTimeout),
    FXMAPFUNC(SEL_COMMAND,          DccTabItem_SPELL,           DccTabItem::onSpellLang),
    FXMAPFUNC(SEL_COMMAND,          dxPipe::ID_PIPE,            DccTabItem::onPipe),
    FXMAPFUNC(SEL_TIMEOUT,          DccTabItem_CTIME,           DccTabItem::onCloseTimeout),
    FXMAPFUNC(SEL_COMMAND,          IrcEngine_SERVER,           DccTabItem::onIrcEvent),
    FXMAPFUNC(SOCKET_CANREAD,       DccTabItem_SOCKET,          DccTabItem::onSocketCanRead),
    FXMAPFUNC(SOCKET_CONNECTED,     DccTabItem_SOCKET,          DccTabItem::onSocketConnected),
    FXMAPFUNC(SOCKET_DISCONNECTED,  DccTabItem_SOCKET,          DccTabItem::onSocketDisconnected),
    FXMAPFUNC(SOCKET_ERR,           DccTabItem_SOCKET,          DccTabItem::onSocketError),
    FXMAPFUNC(SOCKET_STARTACCEPT,   DccTabItem_SOCKET,          DccTabItem::onSocketStartAccept),
    FXMAPFUNC(SOCKET_LISTEN,        DccTabItem_SOCKET,          DccTabItem::onSocketListen),
    FXMAPFUNC(SEL_COMMAND,          LogThread_LIST,             DccTabItem::onListLogThread),
    FXMAPFUNC(SEL_COMMAND,          LogThread_LINES,            DccTabItem::onLinesLogThread)
};

FXIMPLEMENT(DccTabItem, dxTabItem, DccTabItemMap, ARRAYNUMBER(DccTabItemMap))

DccTabItem::DccTabItem(dxTabBook *tab, const FXString &mynick, const FXString &nick,
        const FXString &address, FXint portD, FXint portH, FXbool listen,
        FXIcon *icon, FXuint opts, FXint id,
        FXFont *font, IrcEngine *engine)
    : dxTabItem(tab, nick, icon, opts, id), m_parent(tab), m_listen(listen),
        m_logstream(NULL), m_mynick(mynick), m_nick(nick),
        m_address(address), m_portD(portD), m_portH(portH), m_engine(engine)
{
    m_currentPosition = 0;
    m_historyMax = 25;
    m_pipe = NULL;
    m_sendPipe = FALSE;
    m_scriptHasAll = FALSE;
    m_scriptHasOwnMsg = FALSE;
    m_receiveRest = "";
    m_dccIP = dxUtils.getStringIniEntry("SETTINGS", "dccIP");
    m_dccTimeout = dxUtils.getIntIniEntry("SETTINGS", "dccTimeout", 66);

    m_mainframe = new FXVerticalFrame(m_parent, FRAME_RAISED|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

    m_textframe = new FXVerticalFrame(m_mainframe, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_text->reparent(m_textframe);
    m_text->setFont(font);
    m_text->setSelTextColor(getApp()->getSelforeColor());
    m_text->setSelBackColor(getApp()->getSelbackColor());

    m_commandframe = new FXHorizontalFrame(m_mainframe, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0);
    m_commandline = new dxTextField(m_commandframe, 25, this, DccTabItem_COMMANDLINE, TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|JUSTIFY_LEFT|LAYOUT_FILL_X|LAYOUT_BOTTOM, 0, 0, 0, 0, 1, 1, 1, 1);
    if(preferences.m_sameCmd) m_commandline->setFont(font);
    m_spellLangs = new FXComboBox(m_commandframe, 6, this, DccTabItem_SPELL, COMBOBOX_STATIC);
    m_spellLangs->setTipText(_("Spellchecking language list"));
    m_spellLangs->hide();
    if(preferences.m_sameCmd) m_spellLangs->setFont(font);
    if(preferences.m_useSpell && dxUtils.getAvailableSpellLangsNum())
    {
        dxStringArray langs = dxUtils.getAvailableSpellLangs();
        FXString lang = dxUtils.getChannelLang(getText());
        for(FXint i=0; i<langs.no(); i++)
        {
            m_spellLangs->appendItem(langs[i]);
            if(langs[i]==lang) m_spellLangs->setCurrentItem(i);;
        }
        if(preferences.m_showSpellCombo) m_spellLangs->show();
        m_commandline->setUseSpell(TRUE);
        m_commandline->setLanguage(lang);
        m_commandline->setTipText(FXStringFormat(_("Current spellchecking language: %s"),lang.text()));
    }

    dxHiliteStyle style = {preferences.m_colors.text,preferences.m_colors.back,getApp()->getSelforeColor(),getApp()->getSelbackColor(),0,FALSE};
    for(int i=0; i<19; i++)
    {
        m_textStyleList.append(style);
    }
    //gray text - user commands
    m_textStyleList[0].normalForeColor = preferences.m_colors.user;
    //orange text - Actions
    m_textStyleList[1].normalForeColor = preferences.m_colors.action;
    //blue text - Notice
    m_textStyleList[2].normalForeColor = preferences.m_colors.notice;
    //red text - Errors
    m_textStyleList[3].normalForeColor = preferences.m_colors.error;
    //bold style
    m_textStyleList[4].style = FXText::STYLE_BOLD;
    //underline style
    m_textStyleList[5].style = FXText::STYLE_UNDERLINE;
    //bold & underline
    m_textStyleList[6].style = FXText::STYLE_UNDERLINE;
    m_textStyleList[6].style ^=FXText::STYLE_BOLD;
    //highlight text
    m_textStyleList[7].normalForeColor = preferences.m_colors.hilight;
    //link style
    m_textStyleList[8].normalForeColor = preferences.m_colors.link;
    m_textStyleList[8].link = TRUE;
    //mymsg style
    m_textStyleList[9].normalForeColor = preferences.m_colors.mymsg;
    //log line style
    m_textStyleList[10].normalForeColor = preferences.m_colors.log;
    //next styles for colored nicks
    m_textStyleList[11].normalForeColor = FXRGB(196, 160, 0);
    m_textStyleList[12].normalForeColor = FXRGB(206, 92, 0);
    m_textStyleList[13].normalForeColor = FXRGB(143, 89, 2);
    m_textStyleList[14].normalForeColor = FXRGB(78, 154, 6);
    m_textStyleList[15].normalForeColor = FXRGB(32, 74, 135);
    m_textStyleList[16].normalForeColor = FXRGB(117, 80, 123);
    m_textStyleList[17].normalForeColor = FXRGB(164, 0, 0);
    m_textStyleList[18].normalForeColor = FXRGB(85, 87, 83);

    //text->setStyled(TRUE);
    m_text->setHiliteStyles(m_textStyleList.data());

    m_text->setBackColor(preferences.m_colors.back);
    m_commandline->setBackColor(preferences.m_colors.back);
    m_text->setTextColor(preferences.m_colors.text);
    m_commandline->setTextColor(preferences.m_colors.text);
    m_commandline->setCursorColor(preferences.m_colors.text);
    m_spellLangs->setBackColor(preferences.m_colors.back);
    m_spellLangs->setTextColor(preferences.m_colors.text);

    m_socket = new dxSocket(getApp(), this, DccTabItem_SOCKET);

    m_logThread = new LogThread(this, getApp(), preferences.m_logPath+PATHSEPSTRING+getServerName()+PATHSEPSTRING+getText());
}

DccTabItem::~DccTabItem()
{
    this->stopLogging();
    if(m_pipe) m_pipe->closePipe();
    m_pipeStrings.clear();
    getApp()->removeTimeout(this, DccTabItem_PTIME);
    m_socket->disconnect();
    delete m_socket;
}

void DccTabItem::createGeom()
{
    m_mainframe->create();
    m_commandline->setFocus();
    m_socket->setIsSSL(FALSE);
    if(m_listen)
    {
        IrcEvent ev;
        ev.eventType = IRC_CONNECT;
        ev.param1 = FXStringFormat(_("Listening on %s"), m_address.text());
        ev.time = FXSystem::now();
        writeIrcEvent(ev);
        if(m_socket->isIPv4(m_address))
        {
            if(!m_socket->isRoutableIP(FXUIntVal(m_socket->stringIPToBinary(m_address))))
            {
                ev.eventType = IRC_SERVERERROR;
                ev.param1 = FXStringFormat(_("%s isn't routable."), m_address.text());
                ev.time = FXSystem::now();
                writeIrcEvent(ev);
            }
            if(!m_engine->getMyUserHost().empty() || !m_dccIP.empty())
            {
                if(!m_dccIP.empty())
                {
                    m_address = m_dccIP;
                    ev.eventType = IRC_CONNECT;
                    ev.param1 = FXStringFormat(_("Trying IP from settings %s"), m_address.text());
                    ev.time = FXSystem::now();
                    writeIrcEvent(ev);
                }
                else
                {
                    m_address = m_engine->getMyUserHost();
                    ev.eventType = IRC_CONNECT;
                    ev.param1 = FXStringFormat(_("Trying IP from server %s"), m_address.text());
                    ev.time = FXSystem::now();
                    writeIrcEvent(ev);
                }
                if(m_socket->isIPv4(m_address) && !m_socket->isRoutableIP(FXUIntVal(m_socket->stringIPToBinary(m_address))))
                {
                    ev.eventType = IRC_SERVERERROR;
                    ev.param1 = FXStringFormat(_("%s isn't routable too, but will be used"), m_address.text());
                    ev.time = FXSystem::now();
                    writeIrcEvent(ev);
                }
            }
            else
            {
                ev.eventType = IRC_SERVERERROR;
                ev.param1 = _("But better doesn't exist.");
                ev.time = FXSystem::now();
                writeIrcEvent(ev);
            }
        }
    }
    m_socket->setHost(m_address, m_portD, m_portH);
    if(m_listen) m_socket->listenOn();
    else m_socket->connectTo();
    loadLogLines();
}

void DccTabItem::clearChat()
{
    m_textStyleList.no(19);
    m_text->setHiliteStyles(m_textStyleList.data());
    m_text->clearText();
}

//usefull for set tab current
void DccTabItem::makeLastRowVisible()
{
    m_text->makeLastRowVisible(TRUE);
}

FXString DccTabItem::getSpellLang()
{
#ifdef HAVE_ENCHANT
    if(m_spellLangs->getNumItems()) return m_spellLangs->getItemText(m_spellLangs->getCurrentItem());
#endif
    return "";
}

void DccTabItem::reparentTab()
{
    reparent(m_parent);
    m_mainframe->reparent(m_parent);
    recalc();
    m_mainframe->recalc();
}

void DccTabItem::preferencesUpdated()
{
    setTextForeColor(preferences.m_colors.text);
    setTextBackColor(preferences.m_colors.back);
    setActionsColor(preferences.m_colors.action);
    setNoticeColor(preferences.m_colors.notice);
    setErrorColor(preferences.m_colors.error);
    setHilightColor(preferences.m_colors.hilight);
    setLinkColor(preferences.m_colors.link);
    setMymsgColor(preferences.m_colors.mymsg);
    setLogColor(preferences.m_colors.log);
    stopLogging();
    if(preferences.m_useSpell && dxUtils.getAvailableSpellLangsNum())
    {
        dxStringArray langs = dxUtils.getAvailableSpellLangs();
        FXString lang = dxUtils.getChannelLang(getText());
        for(FXint i=0; i<langs.no(); i++)
        {
            m_spellLangs->appendItem(langs[i]);
            if(langs[i]==lang) m_spellLangs->setCurrentItem(i);;
        }
        if(preferences.m_showSpellCombo) m_spellLangs->show();
        m_commandline->setUseSpell(TRUE);
        m_commandline->setLanguage(lang);
        m_commandline->setTipText(FXStringFormat(_("Current spellchecking language: %s"),lang.text()));
    }
    else
    {
        m_commandline->setUseSpell(FALSE);
        m_commandline->setTipText("");
        m_spellLangs->hide();
    }
    m_commandframe->recalc();
}

void DccTabItem::setTextBackColor(FXColor clr)
{
    for(FXint i=0; i<m_textStyleList.no(); i++)
    {
        m_textStyleList[i].normalBackColor = clr;
    }
    m_text->setBackColor(clr);
    m_commandline->setBackColor(clr);
    m_spellLangs->setBackColor(clr);
}

void DccTabItem::setTextForeColor(FXColor clr)
{
    m_textStyleList[4].normalForeColor = clr;
    m_textStyleList[5].normalForeColor = clr;
    m_textStyleList[6].normalForeColor = clr;
    m_text->setTextColor(clr);
    m_commandline->setTextColor(clr);
    m_commandline->setCursorColor(clr);
    m_spellLangs->setTextColor(clr);
}

void DccTabItem::setActionsColor(FXColor clr)
{
    m_textStyleList[1].normalForeColor = clr;
}

void DccTabItem::setNoticeColor(FXColor clr)
{
    m_textStyleList[2].normalForeColor = clr;
}

void DccTabItem::setErrorColor(FXColor clr)
{
    m_textStyleList[3].normalForeColor = clr;
}

void DccTabItem::setHilightColor(FXColor clr)
{
    m_textStyleList[7].normalForeColor = clr;
}

void DccTabItem::setLinkColor(FXColor clr)
{
    m_textStyleList[8].normalForeColor = clr;
}

void DccTabItem::setMymsgColor(FXColor clr)
{
    m_textStyleList[9].normalForeColor = clr;
}

void DccTabItem::setLogColor(FXColor clr)
{
    m_textStyleList[10].normalForeColor = clr;
}

void DccTabItem::setIrcFont(FXFont *fnt)
{
    m_text->setFont(fnt);
    m_text->recalc();
    if(preferences.m_sameCmd)
    {
        m_commandline->setFont(fnt);
        m_commandline->recalc();
        m_spellLangs->setFont(fnt);
        m_spellLangs->recalc();
    }
    else
    {
        m_commandline->setFont(getApp()->getNormalFont());
        m_commandline->recalc();
        m_spellLangs->setFont(getApp()->getNormalFont());
        m_spellLangs->recalc();
    }
}

void DccTabItem::setSmileys(FXbool smiley, dxSmileyArray nsmileys)
{
    m_text->setSmileys(smiley, nsmileys);
}

void DccTabItem::showSpellComboUpdated()
{
    if(preferences.m_showSpellCombo) m_spellLangs->show();
    else m_spellLangs->hide();
    m_commandframe->recalc();
}

void DccTabItem::removeSmileys()
{
    m_text->removeSmileys();
}

//if highlight==TRUE, highlight tab
void DccTabItem::appendText(FXString msg, FXbool highlight, FXbool logLine)
{
    appendIrcText(msg, 0, FALSE, logLine);
    if(highlight && preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
    {
        if(highlightNeeded(msg))
        {
            this->setTextColor(preferences.m_highlightColor);
        }
        else this->setTextColor(preferences.m_unreadColor);
        this->setIcon(ICO_DCCNEWMSG);
    }
}

void DccTabItem::appendIrcText(FXString msg, FXTime time, FXbool disableStrip, FXbool logLine)
{
    if(!time) time = FXSystem::now();
    m_text->appendText(FXSystem::time("[%H:%M:%S] ", time));
    if(preferences.m_stripColors && !disableStrip) msg = dxutils::stripColors(msg, FALSE);
    appendLinkText(msg, 0);
    if(logLine) this->logLine(msg, time);
}

void DccTabItem::appendIrcNickText(FXString nick, FXString msg, FXint style, FXTime time, FXbool logLine)
{
    if(!time) time = FXSystem::now();
    m_text->appendText(FXSystem::time("[%H:%M:%S] ", time));
    m_text->appendStyledText(nick+": ", style);
    if(preferences.m_stripColors) msg = dxutils::stripColors(msg, FALSE);
    appendLinkText(msg, 0);
    if(logLine) this->logLine("<"+nick+"> "+msg, time);
}

void DccTabItem::appendMyMsg(FXString msg, FXTime time)
{
    if(!time) time = FXSystem::now();
    m_text->appendText(FXSystem::time("[%H:%M:%S] ", time));
    m_text->appendStyledText(getNickName()+": ", preferences.m_coloredNick?10:5);
    if(preferences.m_stripColors) msg = dxutils::stripColors(msg, FALSE);
    appendLinkText(msg, 10);
    this->logLine("\01410\014<"+getNickName()+"> "+msg, time);
}

/* if highlight==TRUE, highlight tab
 * disableStrip is for dxirc.Print
*/
void DccTabItem::appendStyledText(FXString text, FXint style, FXbool highlight, FXbool disableStrip, FXbool logLine)
{
    if(style) appendIrcStyledText(text, style, 0, disableStrip, logLine);
    else appendIrcText(text, 0, disableStrip, logLine);
    if(highlight && preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
    {
        if(highlightNeeded(text))
        {
            this->setTextColor(preferences.m_highlightColor);
        }
        else this->setTextColor(preferences.m_unreadColor);
        this->setIcon(ICO_DCCNEWMSG);
    }
}

void DccTabItem::appendIrcStyledText(FXString styled, FXint stylenum, FXTime time, FXbool disableStrip, FXbool logLine)
{
    if(!time) time = FXSystem::now();
    m_text->appendText(FXSystem::time("[%H:%M:%S] ", time));
    if(preferences.m_stripColors && !disableStrip) styled = dxutils::stripColors(styled, TRUE);
    appendLinkText(styled, stylenum);
    if(logLine)
    {
        if(stylenum==8) //highlight msg
        {
            this->logLine("\0148\014<"+styled.before(':')+">"+styled.after(':'), time);
        }
        else this->logLine(FXStringFormat("\014%d\014%s", stylenum, styled.text()), time);
    }
}

static FXbool isBadchar(FXchar c)
{
    switch(c) {
        case ' ':
        case ',':
        case '\0':
        case '\02':
        case '\03':
        case '\017':
        case '\021':
        case '\026':
        case '\035':
        case '\037':
        case '\n':
        case '\r':
        case '<':
        case '>':
        case '"':
        case '\'':
            return TRUE;
        default:
            return FALSE;
    }
}

// checks is char nick/word delimiter
static FXbool isDelimiter(FXchar c)
{
    switch(c) {
        case ' ':
        case '.':
        case ',':
        case '/':
        case '\\':
        case '`':
        case '\'':
        case '!':
        case '(':
        case ')':
        case '{':
        case '}':
        case '|':
        case '[':
        case ']':
        case '\"':
        case ':':
        case ';':
        case '<':
        case '>':
        case '?':
            return TRUE;
        default:
            return FALSE;
    }
}

void DccTabItem::appendLinkText(const FXString &txt, FXint stylenum)
{
    FXint i = 0;
    FXint linkLength = 0;
    FXbool bold = FALSE;
    FXbool under = FALSE;
    FXint lastStyle = stylenum;
    FXColor foreColor = stylenum && stylenum<=m_textStyleList.no() ? m_textStyleList[stylenum-1].normalForeColor : preferences.m_colors.text;
    FXColor backColor = stylenum && stylenum<=m_textStyleList.no() ? m_textStyleList[stylenum-1].normalBackColor : preferences.m_colors.back;
    FXString normalText = "";
    FXint length = txt.length();
    while(i<length)
    {
        if(txt[i]=='h' && !comparecase(txt.mid(i,7),"http://"))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>7 ? 9 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else if(txt[i]=='h' && !comparecase(txt.mid(i,8),"https://"))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>8 ? 9 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else if(txt[i]=='f' && !comparecase(txt.mid(i,6),"ftp://"))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>6 ? 9 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else if(txt[i]=='w' && !comparecase(txt.mid(i,4),"www."))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>4 ? 9 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else
        {
            if(txt[i] == '\002') //bold
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                bold = !bold;
                FXuint style = 0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                }
            }
            else if(txt[i] == '\026') //reverse
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                FXuint style = 0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                FXColor tempColor = foreColor;
                foreColor = backColor;
                backColor = tempColor;
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                }
            }
            else if(txt[i] == '\037') //underline
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                under = !under;
                FXuint style = 0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                }
            }
            else if(txt[i] == '\021') //fixed
            {
                dxutils::debugLine("Poslan fixed styl");
            }
            else if(txt[i] == '\035') //italic
            {
                dxutils::debugLine("Poslan italic styl");
            }
            else if(txt[i] == '\003') //color
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                FXuint style=0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                FXbool isHexColor = FALSE;
                FXint colorLength = 0;
                foreColor = preferences.m_colors.text;
                backColor = preferences.m_colors.back;
                if(i+1<length)
                {
                    if(txt[i+1] == '#') isHexColor = TRUE;
                }
                if(isHexColor)
                {
                    if(FXRex("^\\h\\h\\h\\h\\h\\h+$").match(txt.mid(i+2,6)))
                    {
                        foreColor = FXRGB(FXIntVal(txt.mid(i+2,2),16),FXIntVal(txt.mid(i+4,2),16),FXIntVal(txt.mid(i+6,2),16));
                        colorLength +=7;
                    }
                    if(i+8 < length && txt[i+8] == ',' && FXRex("^\\h\\h\\h\\h\\h\\h+$").match(txt.mid(i+10,6)))
                    {
                        backColor = FXRGB(FXIntVal(txt.mid(i+10,2),16),FXIntVal(txt.mid(i+12,2),16),FXIntVal(txt.mid(i+14,2),16));
                        colorLength +=8;
                    }
                }
                else
                {
                    if(i+2<length)
                    {
                        FXint code = -1;
                        if(isdigit(txt[i+1]))
                        {
                            if(isdigit(txt[i+2]))
                            {
                                code = (txt[i+1]-48)*10+txt[i+2]-48;
                                colorLength +=2;
                            }
                            else
                            {
                                code = txt[i+1]-48;
                                colorLength ++;
                            }
                        }
                        if(code!=-1)
                            foreColor = dxutils::getIrcColor(code%16);
                    }
                    if(i+colorLength+1 < length && txt[i+colorLength+1] == ',')
                    {
                        FXint code = -1;
                        if(isdigit(txt[i+colorLength+2]))
                        {
                            if(isdigit(txt[i+colorLength+3]))
                            {
                                code = (txt[i+colorLength+2]-48)*10+txt[i+colorLength+3]-48;
                                colorLength +=3;
                            }
                            else
                            {
                                code = txt[i+colorLength+2]-48;
                                colorLength +=2;
                            }
                        }
                        if(code!=-1)
                            backColor = dxutils::getIrcColor(code%16);
                    }
                }
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                };
                i +=colorLength;
            }
            else if(txt[i] == '\017') //reset
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                bold = FALSE;
                under = FALSE;
                foreColor = preferences.m_colors.text;
                backColor = preferences.m_colors.back;
                lastStyle = stylenum;
            }
            else
            {
                normalText.append(txt[i]);
            }
        }
        i++;
    }
    if(!normalText.empty())
    {
        m_text->appendStyledText(normalText, lastStyle);
    }
    m_text->appendText("\n");
}

void DccTabItem::startLogging()
{
    if(m_logstream && FXStat::exists(preferences.m_logPath+PATHSEPSTRING+getServerName()+PATHSEPSTRING+getText()+PATHSEPSTRING+FXSystem::time("%Y-%m-%d", FXSystem::now())))
        return;
    if(preferences.m_logging)
    {
        if(!FXStat::exists(preferences.m_logPath+PATHSEPSTRING+getServerName())) FXDir::create(preferences.m_logPath+PATHSEPSTRING+getServerName());
        if(!FXStat::exists(preferences.m_logPath+PATHSEPSTRING+getServerName()+PATHSEPSTRING+getText())) FXDir::create(preferences.m_logPath+PATHSEPSTRING+getServerName()+PATHSEPSTRING+getText());
        m_logstream = new std::ofstream(FXString(preferences.m_logPath+PATHSEPSTRING+getServerName()+PATHSEPSTRING+getText()+PATHSEPSTRING+FXSystem::time("%Y-%m-%d", FXSystem::now())).text(), std::ios::out|std::ios::app);
    }
}

void DccTabItem::stopLogging()
{
    if(m_logstream)
    {
        m_logstream->close();
        delete m_logstream;
        m_logstream = NULL;
    }
}

void DccTabItem::logLine(const FXString &line, const FXTime &time)
{
    if(preferences.m_logging)
    {
        this->startLogging();
        *m_logstream << FXSystem::time("[%H:%M:%S] ", time).text() << line.text() << std::endl;
    }
}

long DccTabItem::onCommandline(FXObject *, FXSelector, void *)
{
    FXString commandtext = m_commandline->getText();
    if(commandtext.empty())
        return 1;
    m_commandsHistory.append(commandtext);
    if (m_commandsHistory.no() > m_historyMax)
        m_commandsHistory.erase(0);
    m_currentPosition = m_commandsHistory.no()-1;
    m_commandline->setText("");
    if(comparecase(commandtext.left(4),"/say") != 0)
        commandtext.substitute("^B", "\002").substitute("^C", "\003").substitute("^O", "\017").substitute("^V", "\026").substitute("^_", "\037");
    for(FXint i=0; i<=commandtext.contains('\n'); i++)
    {
        FXString text = commandtext.section('\n', i).before('\r');
        if(comparecase(text.after('/').before(' '), "quit") == 0 ||
                comparecase(text.after('/').before(' '), "lua") == 0)
        {
            processLine(text);
            return 1;
        }
#ifdef HAVE_LUA
        if(text[0] != '/') m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, DccTabItem_OWNMSG), &text);
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, DccTabItem_COMMAND), &text);
        if(text[0] == '/' && !m_scriptHasAll) processLine(text);
        else if(!m_scriptHasOwnMsg && !m_scriptHasAll) processLine(text);
#else
        processLine(text);
#endif
    }
    return 1;
}

FXbool DccTabItem::processLine(const FXString& commandtext)
{
//    FXString command = (commandtext[0] == '/' ? commandtext.before(' ') : "");
    // Aliases don't work on dcctabitem
//    if(!utils::instance().getAlias(command).empty())
//    {
//        FXString acommand = utils::instance().getAlias(command);
//        if(acommand.contains("%s"))
//            acommand.substitute("%s", commandtext.after(' '));
//        else
//            acommand += commandtext.after(' ');
//        FXint num = acommand.contains("&&");
//        if(num)
//        {
//            FXbool result = FALSE;
//            for(FXint i=0; i<=acommand.contains('&'); i++)
//            {
//                if(!acommand.section('&',i).trim().empty()) result = processCommand(acommand.section('&',i).trim());
//            }
//            return result;
//        }
//        else
//        {
//            return processCommand(acommand);
//        }
//    }
    return processCommand(commandtext);
}

FXbool DccTabItem::processCommand(const FXString& commandtext)
{
    FXString command = (commandtext[0] == '/' ? commandtext.after('/').before(' ').lower() : "");
    if(dxUtils.isScriptCommand(command))
    {
        LuaRequest lua;
        lua.type = LUA_COMMAND;
        lua.text = commandtext.after('/');
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, DccTabItem_LUA), &lua);
        return TRUE;
    }
    if(command == "commands")
    {
        appendIrcStyledText(dxUtils.availableDccCommands(), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "exec")
    {
        FXString params = commandtext.after(' ');
        if(params.empty())
        {
            appendIrcStyledText(_("/exec [-o|-c] <command>, executes command, -o sends output to channel/query, -c closes running command."), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        else
        {
            if(!m_pipe) m_pipe = new dxPipe(getApp(), this);
            m_pipeStrings.clear();
            if(params.before(' ').contains("-o"))
            {
                m_sendPipe = TRUE;
                m_pipe->execCmd(params.after(' '));
            }
            else if(params.before(' ').contains("-c"))
            {
                m_sendPipe = FALSE;
                m_pipeStrings.clear();
                m_pipe->stopCmd();
            }
            else
            {
                m_sendPipe = FALSE;
                m_pipe->execCmd(params);
            }
            return TRUE;
        }
    }
    if(command == "help")
    {
        return showHelp(commandtext.after(' ').lower().trim());
    }
    if(command == "log")
    {
        FXString logcommand = commandtext.after(' ').before(' ');
        FXString logfile = commandtext.after(' ').after(' ');
        if(comparecase(logcommand, "list")==0)
        {
            listLogFiles();
            return TRUE;
        }
        else if(comparecase(logcommand, "show")==0)
        {
            if(logfile.empty())
                logfile = FXSystem::time("%Y-%m-%d", FXSystem::now());
            FXString logpath = preferences.m_logPath+PATHSEPSTRING+getServerName()+PATHSEPSTRING+getText()+PATHSEPSTRING+logfile;
            if(!FXStat::exists(logpath))
            {
                appendIrcStyledText(FXStringFormat(_("Log file %s doesn't exist"), logfile.text()), 4, FXSystem::now(), FALSE, FALSE);
                return FALSE;
            }
            m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, IrcTabItem_SHOWLOG), &logpath);
            return TRUE;
        }
        else
        {
            appendIrcStyledText(FXStringFormat(_("'%s' isn't <list|show>"), logcommand.text()), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
    }
    if(command == "lua")
    {
#ifdef HAVE_LUA
        FXString luacommand = commandtext.after(' ').before(' ');
        FXString luatext = commandtext.after(' ').after(' ');
        LuaRequest lua;
        if(luacommand.empty())
        {
            appendIrcStyledText(_("/lua <help|load|unload|list> [scriptpath|scriptname]"), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        if(comparecase(luacommand, "help")==0)
        {
            appendIrcStyledText(FXStringFormat(_("For help about Lua scripting visit: %s"), LUA_HELP_PATH), 3, FXSystem::now(), FALSE, FALSE);
            return TRUE;
        }
        else if(comparecase(luacommand, "load")==0) lua.type = LUA_LOAD;
        else if(comparecase(luacommand, "unload")==0) lua.type = LUA_UNLOAD;
        else if(comparecase(luacommand, "list")==0) lua.type = LUA_LIST;
        else
        {
            appendIrcStyledText(FXStringFormat(_("'%s' isn't <help|load|unload|list>"), luacommand.text()), 4, FXSystem::now(), FALSE, FALSE);
            return FALSE;
        }
        lua.text = luatext;
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, DccTabItem_LUA), &lua);
        return TRUE;
#else
        appendIrcStyledText(_("dxirc is compiled without support for Lua scripting"), 4, FXSystem::now(), FALSE, FALSE);
        return FALSE;
#endif
    }
    if(command == "me")
    {
        if(m_socket->isConnected())
        {
            appendIrcStyledText(getNickName()+" "+commandtext.after(' '), 2, FXSystem::now());
            return sendLine("\001ACTION "+commandtext.after(' ')+"\001\n");
        }
        else
        {
            appendIrcStyledText(_("You aren't connected"), 4, FXSystem::now(), FALSE, FALSE);
            return TRUE;
        }
    }
    if(command == "say")
    {
        if(m_socket->isConnected())
        {
            appendMyMsg(commandtext.after(' '), FXSystem::now());
            return sendLine(commandtext.after(' ')+"\n");
        }
        else
        {
            appendIrcStyledText(_("You aren't connected"), 4, FXSystem::now(), FALSE, FALSE);
            return TRUE;
        }
    }
    if(command == "tetris")
    {
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, DccTabItem_NEWTETRIS), NULL);
        return TRUE;
    }
    if(commandtext[0] != '/')
    {
        if(m_socket->isConnected())
        {
            appendMyMsg(commandtext, FXSystem::now());
            return sendLine(commandtext+"\n");
        }
        else
        {
            appendIrcStyledText(_("You aren't connected"), 4, FXSystem::now(), FALSE, FALSE);
            return TRUE;
        }
    }
    return FALSE;
}

FXbool DccTabItem::showHelp(FXString command)
{
    if(dxUtils.isScriptCommand(command))
    {
        appendIrcStyledText(dxUtils.getHelpText(command), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "commands")
    {
        appendIrcStyledText(_("COMMANDS, shows available commands"), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
#ifndef WIN32
    if(command == "exec")
    {
        appendIrcStyledText(_("EXEC [-o|-c] <command>, executes command, -o sends output to channel/query, -c closes running command."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
#endif
    if(command == "help")
    {
        appendIrcStyledText(_("HELP <command>, shows help for command."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "log")
    {
        appendIrcStyledText(_("LOG list, shows list of log files for current tab"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("LOG show [date], shows last log file for current tab, if date is empty"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("else file for given date"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("date example: 2011-01-31"), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
#ifdef HAVE_LUA
    if(command == "lua")
    {
        appendIrcStyledText(_("LUA help, shows help for lua scripting."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("LUA load <path>, loads script."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("Example: /lua load /home/dvx/test.lua"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("LUA unload <name>, unloads script."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("Example: /lua unload test"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("LUA list, shows list of loaded scripts"), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
#else
    if(command == "lua")
    {
        appendIrcStyledText(_("dxirc is compiled without support for Lua scripting"), 4, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
#endif
    if(command == "me")
    {
        appendIrcStyledText(_("ME <to> <message>, sends the action."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "say")
    {
        appendIrcStyledText(_("SAY [text], sends text to current tab."), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command == "tetris")
    {
        appendIrcStyledText(_("TETRIS, start small easteregg."), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("Keys for playing:"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("n .. new game"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("p .. pause game"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("i .. rotate piece"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("l .. move piece right"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("k .. drop piece"), 3, FXSystem::now(), FALSE, FALSE);
        appendIrcStyledText(_("j .. move piece left"), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(!dxUtils.getAlias(command[0] == '/' ? command:"/"+command).empty())
    {
        appendIrcStyledText(FXStringFormat("%s: %s", command.upper().text(), dxUtils.getAlias(command[0] == '/' ? command:"/"+command).text()), 3, FXSystem::now(), FALSE, FALSE);
        return TRUE;
    }
    if(command.empty()) appendIrcStyledText(_("Command is empty, type /commands for available commands"), 4, FXSystem::now(), FALSE, FALSE);
    else appendIrcStyledText(FXStringFormat(_("Unknown command '%s', type /commands for available commands"), command.text()), 4, FXSystem::now(), FALSE, FALSE);
    return FALSE;
}

long DccTabItem::onKeyPress(FXObject *, FXSelector, void *ptr)
{
    if (m_commandline->hasFocus())
    {
        FXString line = m_commandline->getText();
        FXString key = dxutils::getKeybinding((FXEvent*)ptr);
        if(comparecase(key, "Tab")==0) //tab completion
        {
            if(comparecase(line.before(' '), "/help") == 0 && !line.after(' ').empty())
            {
                completeHelpCommand();
                return 1;
            }
            if(line[0] == '/' && line.after(' ').empty())
            {
                for(FXint i = 0; i < dxUtils.dcccommandsNo(); i++)
                {
                    if(comparecase(line.after('/').before(' '), dxUtils.dcccommandsAt(i)) == 0)
                    {
                        if((i+1) < dxUtils.dcccommandsNo()) m_commandline->setText("/"+dxUtils.dcccommandsAt(++i)+" ");
                        else m_commandline->setText("/"+dxUtils.dcccommandsAt(0)+" ");
                        break;
                    }
                    else if(comparecase(line.after('/'), dxUtils.dcccommandsAt(i).left(line.after('/').length())) == 0)
                    {
                        m_commandline->setText("/"+dxUtils.dcccommandsAt(i)+" ");
                        break;
                    }
                }
                return 1;
            }
            return 1;
        }
        else if(comparecase(key, "Up")==0) //append history command backward
        {
            if(m_currentPosition!=-1 && m_currentPosition<m_commandsHistory.no())
            {
                if(!line.empty() && line!=m_commandsHistory[m_currentPosition])
                {
                    m_commandsHistory.append(line);
                    if(m_commandsHistory.no() > m_historyMax)
                        m_commandsHistory.erase(0);
                    m_currentPosition = m_commandsHistory.no()-1;
                }
                if(m_currentPosition > 0 && !line.empty())
                    --m_currentPosition;
                m_commandline->setText(m_commandsHistory[m_currentPosition]);
            }
            return 1;
        }
        else if(comparecase(key, "Down")==0) //append history command forward
        {
            if(m_currentPosition!=-1 && m_currentPosition<m_commandsHistory.no())
            {
                if(!line.empty() && line!=m_commandsHistory[m_currentPosition])
                {
                    m_commandsHistory.append(line);
                    if(m_commandsHistory.no() > m_historyMax)
                        m_commandsHistory.erase(0);
                    m_currentPosition = m_commandsHistory.no()-1;
                }
                if(m_currentPosition < m_commandsHistory.no()-1)
                {
                    ++m_currentPosition;
                    m_commandline->setText(m_commandsHistory[m_currentPosition]);
                }
                else
                    m_commandline->setText("");
            }
            return 1;
        }
    }
    return 0;
}

long DccTabItem::onPipe(FXObject*, FXSelector, void *ptr)
{
    if(!m_socket->isConnected())
    {
        m_sendPipe = FALSE;
        m_pipeStrings.clear();
        m_pipe->stopCmd();
        return 1;
    }
    FXString text = *(FXString*)ptr;
    if(m_sendPipe)
    {
        if(!getApp()->hasTimeout(this, DccTabItem_PTIME)) getApp()->addTimeout(this, DccTabItem_PTIME);
        m_pipeStrings.append(text);
    }
    else appendIrcText(text, FXSystem::now(), TRUE, FALSE);
    return 1;
}

long DccTabItem::onIrcEvent(FXObject *, FXSelector, void *data)
{
    IrcEvent *ev = (IrcEvent *)data;
    if(ev->eventType == IRC_NICK)
    {
        if(m_mynick==ev->param1)
        {
            m_mynick = ev->param2;
            appendIrcStyledText(FXStringFormat(_("You're now known as %s"), m_mynick.text()), 1, ev->time, FALSE, FALSE);
        }
        if(m_nick==ev->param1)
        {
            m_nick = ev->param2;
            appendIrcStyledText(FXStringFormat(_("%s is now known as %s"), ev->param1.text(), ev->param2.text()), 1, ev->time, FALSE, FALSE);
            setText(m_nick);
            stopLogging();
        }
        return 1;
    }
    return 1;
}

//handle LogThread_LIST
long DccTabItem::onListLogThread(FXObject*, FXSelector, void*)
{
    dxStringArray files = m_logThread->getResult();
    if(!files.no())
    {
        appendIrcStyledText(FXStringFormat(_("No log files for %s-%s."), m_engine->getServerName().text(), getText().text()), 4, FXSystem::now(), FALSE, FALSE);
        return 1;
    }
    appendIrcStyledText(_("Log files:"), 7, FXSystem::now(), FALSE, FALSE);
    FXString lastYear = dxutils::getParam(files[0], 1, FALSE, '-');
    FXint lastMonth = FXIntVal(dxutils::getParam(files[0], 2, FALSE, '-'));
    appendIrcStyledText(FXStringFormat("%s:", lastYear.text()), 5, FXSystem::now(), FALSE, FALSE);
    appendIrcStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 5, FXSystem::now(), FALSE, FALSE);
    FXString list = "";
    for(FXint i=0; i<files.no(); i++)
    {
        if(lastYear!=dxutils::getParam(files[i], 1, FALSE, '-'))
        {
            lastYear = dxutils::getParam(files[i], 1, FALSE, '-');
            lastMonth = FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-'));
            appendIrcText(list.rbefore(','), FXSystem::now(), FALSE, FALSE);
            appendIrcStyledText(FXStringFormat("%s:", lastYear.text()), 5, FXSystem::now(), FALSE, FALSE);
            appendIrcStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 5, FXSystem::now(), FALSE, FALSE);
            list.clear();
            list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
        }
        else if(lastMonth!=FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-')))
        {
            lastMonth = FXIntVal(dxutils::getParam(files[i], 2, FALSE, '-'));
            appendIrcText(list.rbefore(','), FXSystem::now(), FALSE, FALSE);
            appendIrcStyledText(FXStringFormat("%s:", dxutils::getMonth(lastMonth).text()), 5, FXSystem::now(), FALSE, FALSE);
            list.clear();
            list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
        }
        else list.append(dxutils::getParam(files[i], 3, FALSE, '-')+", ");
    }
    if(!list.empty()) appendIrcText(list.rbefore(','), FXSystem::now(), FALSE, FALSE);
    return 1;
}

//handle LogThread_LINES
long DccTabItem::onLinesLogThread(FXObject*, FXSelector, void*)
{
    clearChat();
    dxStringArray lines = m_logThread->getResult();
    for(FXint i=0; i<lines.no(); i++)
    {
        if(lines[i].after(']')[1]=='\014') //log style mark
        {
            if(lines[i].after('\014',2)[0]=='<') //has nick
            {
                m_text->appendStyledText(lines[i].before('\014'), 11); //timestamp
                FXString nick = lines[i].after('<').before('>');
                m_text->appendStyledText(nick+":", getNickStyle(nick));
                appendLinkText(dxutils::stripColors(lines[i].after('>'), TRUE), 11);
            }
            else appendLinkText(dxutils::stripLogMark(dxutils::stripColors(lines[i], TRUE)), 11);
        }
        else
        {
            if(lines[i].after(']')[1]=='<') //has nick
            {
                m_text->appendStyledText(lines[i].before('<'), 11); //timestamp
                FXString nick = lines[i].after('<').before('>');
                m_text->appendStyledText(nick+":", getNickStyle(nick));
                appendLinkText(dxutils::stripColors(lines[i].after('>'), TRUE), 11);
            }
            else appendLinkText(dxutils::stripColors(lines[i], TRUE), 11);
        }
    }
    return 1;
}

long DccTabItem::onPipeTimeout(FXObject*, FXSelector, void*)
{
    if(m_pipeStrings.no() > 3)
    {
        if(m_pipeStrings[0].empty()) m_pipeStrings[0] = " ";
#ifdef HAVE_LUA
        m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, DccTabItem_OWNMSG), &m_pipeStrings[0]);
        if(!m_scriptHasOwnMsg)
        {
            appendMyMsg(m_pipeStrings[0], FXSystem::now());
            sendLine(m_pipeStrings[0]+"\n");
        }
#else
        appendMyMsg(m_pipeStrings[0], FXSystem::now());
        sendLine(m_pipeStrings[0]+"\n");
#endif
        m_pipeStrings.erase(0);
        getApp()->addTimeout(this, DccTabItem_PTIME, 3000);
    }
    else
    {
        while(m_pipeStrings.no())
        {
            if(m_pipeStrings[0].empty()) m_pipeStrings[0] = " ";
#ifdef HAVE_LUA
            m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, DccTabItem_OWNMSG), &m_pipeStrings[0]);
            if(!m_scriptHasOwnMsg)
            {
                appendMyMsg(m_pipeStrings[0], FXSystem::now());
                sendLine(m_pipeStrings[0]+"\n");
            }
#else
            appendMyMsg(m_pipeStrings[0], FXSystem::now());
            sendLine(m_pipeStrings[0]+"\n");
#endif
            m_pipeStrings.erase(0);
        }
    }
    return 1;
}

//for offered connection by me
long DccTabItem::onCloseTimeout(FXObject*, FXSelector, void*)
{
    if(!m_socket->isConnected())
    {
        appendIrcStyledText(_("Connection closed. Client didn't connect in given timeout"), 4, FXSystem::now(), TRUE, FALSE);
        m_socket->disconnect();
    }
    return 1;
}

//handle change in spellLang combobox
long DccTabItem::onSpellLang(FXObject*, FXSelector, void*)
{
    m_commandline->setLanguage(m_spellLangs->getItemText(m_spellLangs->getCurrentItem()));
    m_commandline->setTipText(FXStringFormat(_("Current spellchecking language: %s"),m_spellLangs->getItemText(m_spellLangs->getCurrentItem()).text()));
    return 1;
}

FXint DccTabItem::getNickStyle(const FXString &nick)
{
    if(preferences.m_coloredNick) return 12+nick.hash()%8; //12 is first colored nick style
    return 5; //bold
}

FXint DccTabItem::hiliteStyleExist(FXColor foreColor, FXColor backColor, FXuint style)
{
    for(FXint i=0; i<m_textStyleList.no(); i++)
    {
        if(m_textStyleList[i].normalForeColor == foreColor
                && m_textStyleList[i].normalBackColor == backColor
                && m_textStyleList[i].style == style)
            return i+1;
    }
    return -1;
}

void DccTabItem::createHiliteStyle(FXColor foreColor, FXColor backColor, FXuint style)
{
    dxHiliteStyle nstyle = {foreColor,backColor,getApp()->getSelforeColor(),getApp()->getSelbackColor(),style,FALSE};
    m_textStyleList.append(nstyle);
    m_text->setHiliteStyles(m_textStyleList.data());
}

dxStringArray DccTabItem::cutText(FXString text, FXint len)
{
    FXint textLen = text.length();
    FXint previous = 0;
    dxStringArray texts;
    while(textLen>len)
    {
        texts.append(text.mid(previous, len));
        previous += len;
        textLen -= len;
    }
    texts.append(text.mid(previous, len));
    return texts;
}

void DccTabItem::setCommandFocus()
{
    m_commandline->setFocus();
}

//for "handle" checking, if script contains "all". Send from dxirc.
void DccTabItem::setHasAllCommand(FXbool hasAll)
{
    m_scriptHasAll = hasAll;
}

//for "handle" checking, if script contains "ownmsg". Send from dxirc.
void DccTabItem::setHasOwnMsg(FXbool hasOwn)
{
    m_scriptHasOwnMsg = hasOwn;
}

//check need of highlight in msg
FXbool DccTabItem::highlightNeeded(const FXString &msg)
{
    FXint pos = msg.find(getNickName());
    if(pos==-1) return FALSE;
    FXbool before = TRUE;
    FXbool after = FALSE;
    if(pos) before = isDelimiter(msg[pos-1]);
    if(pos+getNickName().length() == msg.length()) after = TRUE;
    if(pos+getNickName().length() < msg.length()) after = isDelimiter(msg[pos+getNickName().length()]);
    return before && after;
}

FXbool DccTabItem::sendLine(const FXString& line)
{
    dxutils::debugLine(FXStringFormat(">> %s", line.before('\n').text()));
    FXint size=0;
    if(m_socket->isConnected())
    {
        size = m_socket->writeLine(line);
        if(size != -1) return TRUE;
        else return FALSE;
    }
    else return FALSE;
}

void DccTabItem::writeIrcEvent(IrcEvent &event)
{
    switch(event.eventType) {
        case IRC_DCCMSG:
        {
            FXbool needHighlight = FALSE;
            if(event.param1.contains(m_mynick))
                needHighlight = highlightNeeded(event.param1);
            if(needHighlight) appendIrcStyledText(m_nick+": "+event.param1, 8, event.time);
            else appendIrcNickText(m_nick, event.param1, getNickStyle(m_nick), event.time);
            if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
            {
                if(needHighlight)
                {
                    this->setTextColor(preferences.m_highlightColor);
                }
                else this->setTextColor(preferences.m_unreadColor);
                this->setIcon(ICO_DCCNEWMSG);
                m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, DccTabItem_NEWMSG), NULL);
            }
        }break;
        case IRC_DCCACTION:
        {
            appendIrcStyledText(m_nick+" "+event.param1, 2, event.time);
            if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this))
            {
                if(event.param1.contains(m_mynick) && highlightNeeded(event.param1))
                {
                    this->setTextColor(preferences.m_highlightColor);
                }
                else this->setTextColor(preferences.m_unreadColor);
                this->setIcon(ICO_DCCNEWMSG);
                m_parent->getParent()->getParent()->handle(this, FXSEL(SEL_COMMAND, DccTabItem_NEWMSG), NULL);
            }
        }break;
        case IRC_CONNECT:
        {
            appendIrcStyledText(event.param1, 3, event.time, TRUE, FALSE);
        }break;
        case IRC_DISCONNECT:
        case IRC_SERVERERROR:
        {
            appendIrcStyledText(event.param1, 4, event.time, TRUE, FALSE);
            if(preferences.m_highlightColor != this->getTextColor() && m_parent->getCurrent()*2 != m_parent->indexOfChild(this)) this->setTextColor(preferences.m_unreadColor);
        }break;
        default:
            break;
    }
}

long DccTabItem::onSocketCanRead(FXObject*, FXSelector, void*)
{
    FXchar buffer[1024];
    FXString data = m_receiveRest;
    FXint size = 0;
    while((size = m_socket->read(buffer, 1023)) > 0)
    {
        buffer[size] = '\0';
        if(dxutils::isUtf8(buffer, size)) data.append(buffer);
        else data.append(dxUtils.localeToUtf8(buffer));
        while(data.contains('\n'))
        {
            IrcEvent ev;
            ev.param1 = data.before('\n').before('\r');
            ev.time = FXSystem::now();
            if(ev.param1.contains('\001'))
            {
                ev.eventType = IRC_DCCACTION;
                ev.param1 = ev.param1.after('\001').before('\001').after(' ');
            }
            else
                ev.eventType = IRC_DCCMSG;
            dxutils::debugLine(FXStringFormat("<< %s",ev.param1.text()));
            writeIrcEvent(ev);
            data = data.after('\n');
        }
        m_receiveRest = data;
    }
    return 1;
}

long DccTabItem::onSocketConnected(FXObject*, FXSelector, void*)
{
    IrcEvent ev;
    ev.eventType = IRC_CONNECT;
    ev.param1 = FXStringFormat(_("Connected to %s"), m_address.text());
    ev.time = FXSystem::now();
    writeIrcEvent(ev);
    return 1;
}

long DccTabItem::onSocketDisconnected(FXObject*, FXSelector, void*)
{
    m_sendPipe = FALSE;
    m_pipeStrings.clear();
    if(m_pipe) m_pipe->stopCmd();
    IrcEvent ev;
    ev.eventType = IRC_DISCONNECT;
    ev.param1 = FXStringFormat(_("Server %s was disconnected"), m_address.text());
    ev.time = FXSystem::now();
    writeIrcEvent(ev);
    return 1;
}

long DccTabItem::onSocketError(FXObject*, FXSelector, void *ptr)
{
    m_sendPipe = FALSE;
    m_pipeStrings.clear();
    if(m_pipe) m_pipe->stopCmd();
    SocketError *err = NULL;
    if(ptr) err = *((SocketError**)ptr);
    IrcEvent ev;
    ev.eventType = IRC_SERVERERROR;
    ev.time = FXSystem::now();
    if(err)
    {
        if(!err->errorStr.empty())
        {
            ev.param1 = err->errorStr;
            writeIrcEvent(ev);
        }
        switch(err->errorType){
            case UNABLEINIT:
            {
                ev.param1 = _("Unable to initiliaze socket");
                writeIrcEvent(ev);
                break;
            }
            case BADHOST:
            {
                ev.param1 = FXStringFormat(_("Bad host: %s"), m_address.text());
                writeIrcEvent(ev);
                break;
            }
            case UNABLECREATE:
            {
                ev.param1 = _("Unable to create socket");
                writeIrcEvent(ev);
                break;
            }
            case UNABLECONNECT:
            {
                ev.param1 = FXStringFormat(_("Unable to connect to: %s"), m_address.text());
                writeIrcEvent(ev);
                break;
            }
            case SSLUNCREATE:
            {
                ev.param1 = _("SSL creation error");
                writeIrcEvent(ev);
                break;
            }
            case SSLCONNECTERROR:
            {
                ev.param1 = _("SSL connect error");
                writeIrcEvent(ev);
                break;
            }
            case UNABLEBIND:
            {
                ev.param1 = _("Unable to bind socket");
                writeIrcEvent(ev);
                break;
            }
            case UNABLELISTEN:
            {
                ev.param1 = _("Unable to listen");
                writeIrcEvent(ev);
                break;
            }
            case UNABLEACCEPT:
            {
                ev.param1 = _("Unable to accept connection");
                writeIrcEvent(ev);
                break;
            }
            case UNABLESEND:
            {
                ev.param1 = _("Unable to send data");
                writeIrcEvent(ev);
                break;
            }
            case UNABLEREAD:
            {
                ev.param1 = FXStringFormat(_("Error in reading data from %s"), m_address.text());
                writeIrcEvent(ev);
                break;
            }
            case SSLZERO:
            {
                ev.param1 = _("SSL_read() returns zero - closing socket");
                writeIrcEvent(ev);
                break;
            }
            case SSLUNABLEREAD:
            {
                ev.param1 = _("SSL read problem");
                writeIrcEvent(ev);
                break;
            }
            case SSLABNORMAL:
            {
                ev.param1 = _("Abnormal value from SSL read");
                writeIrcEvent(ev);
                break;
            }
            case UNABLEREADBUFFER:
            {
                ev.param1 = _("Unable read data from buffer");
                writeIrcEvent(ev);
                break;
            }
            default:
                break;
        }
        delete err;
    }
    return 1;
}

long DccTabItem::onSocketStartAccept(FXObject*, FXSelector, void*)
{
    m_portD = m_socket->getPort();
    if(m_engine && m_listen) m_engine->sendCtcp(m_nick, "DCC CHAT chat "+m_socket->stringIPToBinary(m_address)+" "+FXStringVal(m_portD));
    getApp()->addTimeout(this, DccTabItem_CTIME, m_dccTimeout*1000);
    return 1;
}

long DccTabItem::onSocketListen(FXObject*, FXSelector, void*)
{
    IrcEvent ev;
    ev.eventType = IRC_CONNECT;
    ev.param1 = FXStringFormat(_("Someone connected from %s"), m_socket->getRemoteIP().text());
    ev.time = FXSystem::now();
    writeIrcEvent(ev);
    getApp()->removeTimeout(this, DccTabItem_CTIME);
    return 1;
}

//Load lines from logs
void DccTabItem::loadLogLines()
{
    if(!preferences.m_loadLog)
        return;
    if(m_logThread->running())
    {
        appendStyledText(_("Thread running, try again after some seconds"), 7, false, false, false);
        return;
    }
    appendIrcStyledText(_("Loading log..."), 11, FXSystem::now(), FALSE, FALSE);
    m_logThread->setList(false);
    m_logThread->setDirectory(preferences.m_logPath+PATHSEPSTRING+getServerName()+PATHSEPSTRING+getText());
    m_logThread->start();
}

//List log files for tab
void DccTabItem::listLogFiles()
{
    if(m_logThread->running())
    {
        appendStyledText(_("Thread running, try again after some seconds"), 7, false, false, false);
        return;
    }
    m_logThread->setList(true);
    m_logThread->setDirectory(preferences.m_logPath+PATHSEPSTRING+getServerName()+PATHSEPSTRING+getText());
    m_logThread->start();
}

void DccTabItem::completeHelpCommand()
{
    FXString line = m_commandline->getText();
    for(FXint i = 0; i < dxUtils.dcccommandsNo(); i++)
    {
        if(comparecase(line.after(' '), dxUtils.dcccommandsAt(i)) == 0)
        {
            if((i+1) < dxUtils.dcccommandsNo()) m_commandline->setText("/HELP "+dxUtils.dcccommandsAt(++i));
            else m_commandline->setText("/HELP "+dxUtils.dcccommandsAt(0));
            break;
        }
        else if(comparecase(line.after(' '), dxUtils.dcccommandsAt(i).left(line.after(' ').length())) == 0)
        {
            m_commandline->setText("/HELP "+dxUtils.dcccommandsAt(i));
            break;
        }
    }
}
