/*
 *      keydialog.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "keydialog.h"
#include "preferences.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

BindEditDialog::BindEditDialog(const QString& command, const QKeySequence &sequence, QWidget *parent)
: QDialog(parent), m_sequence(sequence)
{
    QVBoxLayout *mainframe = new QVBoxLayout(this);
    mainframe->addWidget(new QLabel(command));
    m_bind = new QLabel(sequence.toString(QKeySequence::NativeText));
    mainframe->addWidget(m_bind);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    mainframe->addWidget(buttons);
    setLayout(mainframe);
    setWindowTitle(_("Modify Key binding"));
    m_bind->setFocus();
}

void BindEditDialog::keyPressEvent(QKeyEvent* e)
{
    m_sequence = dxutils::getKeybinding(e);
    m_bind->setText(m_sequence.toString(QKeySequence::NativeText));
    m_bind->setFocus();
}

QKeySequence BindEditDialog::sequence() const
{
    return m_sequence;
}

KeyDialog::KeyDialog(QWidget *parent)
: QDialog(parent)
{
    m_binds = preferences.m_keybindings;
    m_changed = false;
    m_restart = false;
    m_nonchangeableBindings.append("Tab");
    m_nonchangeableBindings.append("Up");
    m_nonchangeableBindings.append("Down");
    m_nonchangeableBindings.append("F3");
    m_nonchangeableBindings.append("N");
    m_nonchangeableBindings.append("P");
    m_nonchangeableBindings.append("I");
    m_nonchangeableBindings.append("J");
    m_nonchangeableBindings.append("K");
    m_nonchangeableBindings.append("L");

    QVBoxLayout *mainFrame = new QVBoxLayout(this);

    QStringList headers;
    headers << _("Command") << _("Key binding");
    m_table = new QTreeWidget;
    m_table->setHeaderLabels(headers);
    m_table->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_table->setAlternatingRowColors(true);
    m_table->setRootIsDecorated(false);
    m_table->setSelectionMode(QAbstractItemView::SingleSelection);
    connect(m_table, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), this, SLOT(slotTable(QTreeWidgetItem*)));
    QHeaderView *header = m_table->header();
    header->resizeSection(0, 300);
    header->resizeSection(1, 100);
    mainFrame->addWidget(m_table);

    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Cancel);
    m_buttonSaveClose = new QPushButton(_("&Save&&Close"));
    connect(m_buttonSaveClose, SIGNAL(clicked()), this, SLOT(slotSaveClose()));
    buttons->addButton(m_buttonSaveClose, QDialogButtonBox::ActionRole);
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    mainFrame->addWidget(buttons);

    resize(500,400);
    setLayout(mainFrame);
    setWindowTitle(_("Key bindings list"));

    updateTable();
}

void KeyDialog::slotTable(QTreeWidgetItem *item)
{
    if(item)
    {
        QString key;
        QString value = item->text(1);
        QKeySequence sequence;
        QMap<QString,QKeySequence>::iterator it;
        for(it=m_binds.begin(); it!=m_binds.end(); it++)
        {
            if(it.value().toString(QKeySequence::NativeText) == value)
            {
                key = it.key();
                sequence = it.value();
                break;
            }
        }
        QString message = _("Press the combination of keys you want to use for the action: %1").arg(item->text(0));
        message = message+ "\n" + _("[Press space to disable the key binding for this action]");
        BindEditDialog *dialog = new BindEditDialog(message, sequence, this);
        if(dialog->exec() == QDialog::Accepted)
        {
            sequence = dialog->sequence();
            if(sequence.toString() == "Space") sequence = QKeySequence();
            if(!bindingExist(sequence))
            {
                m_changed = true;
                m_binds.remove(key);
                m_binds.insert(key, sequence);
                updateTable();
            }
            else
            {
                QMessageBox::warning(this, _("Error"), _("The key binding %1 is already used.\nYou should erase the existing key binding before assigning it again.").arg(value), QMessageBox::Ok);
            }
        }
    }
}

void KeyDialog::slotSaveClose()
{
    if(m_changed)
    {
        preferences.m_keybindings = m_binds;
        if(QMessageBox::question(this, _("Restart"), _("Key bindings will be changed after restart.\nRestart dxirc now?"), QMessageBox::Ok|QMessageBox::No)==QMessageBox::Ok)
        {
            m_restart = true;
        }
        m_changed = false;
    }
    accept();
}

void KeyDialog::updateTable()
{
    m_table->clear();
    QMap<QString,QKeySequence>::iterator it;
    for(it=m_binds.begin(); it!=m_binds.end(); it++)
    {
        QTreeWidgetItem *item = new QTreeWidgetItem(m_table);
        item->setText(0, preferences.getBindingName(it.key()));
        item->setText(1, it.value().toString(QKeySequence::NativeText));
    }
    m_table->sortByColumn(0,Qt::AscendingOrder);
}

bool KeyDialog::bindingExist(const QKeySequence& sequence)
{
    if(sequence.isEmpty()) return false;
    for(int i=0; i<m_nonchangeableBindings.size(); i++)
    {
        if(m_nonchangeableBindings[i].compare(sequence.toString(QKeySequence::NativeText), Qt::CaseInsensitive)==0)
            return true;
    }
    QMap<QString,QKeySequence>::iterator it;
    for(it=m_binds.begin(); it!=m_binds.end(); it++)
    {
        if(it.value() == sequence) return true;
    }
    return false;
}
