/*
 *      ircengine.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#ifndef IRCENGINE_H
#define IRCENGINE_H

#include <QTimer>
#include "defs.h"
#include "config.h"
#include "dxutils.h"
#ifndef DXSOCKET_H
#include "dxsocket.h"
#endif

class IrcEngine: public QObject
{
    Q_OBJECT
public:
    IrcEngine(QString, QString);
    virtual ~IrcEngine();

    void startConnection();
    void disconnectIrc();
    void disconnectIrc(const QString &reason);
    void reconnect();
    void closeConnection(bool disableReconnect=false);
    void clearAttempts() { m_attempts = 0; }
    void setServerName(const QString &name) { m_serverName = name; }
    QString getServerName() const { return m_serverName; }
    QString getNetworkName() const { return m_networkName; }
    void setServerPort(int port) { m_serverPort = port; }
    int getServerPort() const { return m_serverPort; }
    void setServerPassword(const QString &pass) { m_serverPassword = pass; }
    void setNickName(const QString &nick) { m_nickName = nick; }
    QString getNickName() const { return m_nickName; }
    void setUserName(const QString &user);
    void setRealName(const QString &realname) { m_realName = realname; }
    void setStartChannels(const QString &channels) { m_startChannels = dxutils::removeSpaces(channels); }
    void setStartCommands(const QString &commands) { m_startCommands = commands; }
    void setUseSsl(const bool &usessl) { m_useSsl = usessl; }
    bool getConnected() const { return m_connected; }
    bool getConnecting() const { return m_connecting; }
    bool getUseSsl() const { return m_useSsl; }
    int getNickLen() const { return m_nickLen; }
    int getTopicLen() const { return m_topicLen; }
    int getKickLen() const { return m_kickLen; }
    int getAwayLen() const { return m_awayLen; }
    char getAdminPrefix() const { return m_adminPrefix; }
    char getOwnerPrefix() const { return m_ownerPrefix; }
    char getOpPrefix() const { return m_opPrefix; }
    char getVoicePrefix() const { return m_voicePrefix; }
    char getHalfopPrefix() const { return m_halfopPrefix; }
    QString getChanTypes() const { return m_chanTypes; }
    QString getLocalIP() { return m_socket->getLocalIP(); }
    QString getLocalIPBinary();
    QString getHostname(const QString &address);
    QString getRemoteIP()  { return m_socket->getRemoteIP(); }
    void addIgnoreWho(const QString &who);
    void addNick(const QString &nick, const QString &user, const QString &real, const QString &host, const bool &away);
    void addNick(const QString &nick, const QString &user, const QString &real, const QString &host);
    void removeNick(const QString &nick);
    NickInfo getNickInfo(const QString &nick) const;
    QStringList getBannedNick(const QString &banmask) const;
    QStringList getChannelsList() const;
    void clearChannelsList();
    bool sendAdmin(const QString &params);
    bool sendAway(const QString &params);
    bool sendBanlist(const QString &channel);
    bool sendCtcp(const QString &to, const QString &params);
    bool sendCtcpNotice(const QString &to, const QString &params);
    bool sendMode(const QString &params);
    bool sendInvite(const QString &to, const QString &params);
    bool sendJoin(const QString &chan);
    bool sendKick(const QString &chan, const QString &nick, const QString &reason);
    bool sendKill(const QString &nick, const QString &reason);
    bool sendList(const QString &params);
    bool sendMe(const QString &to, const QString &message);
    bool sendMsg(const QString &to, const QString &message);
    /** Send names request
     * 
     * @param channel   name of channel
     * @param list      true if command answer could be show
     * @return          true is command was succesfull
     */
    bool sendNames(const QString &channel, bool list);
    bool sendNick(const QString &nick);
    bool sendNotice(const QString &to, const QString &message);
    bool sendOper(const QString &login, const QString &password);
    bool sendPart(const QString &chan);
    bool sendPart(const QString &chan, const QString &reason);
    bool sendQuote(const QString &text);
    bool sendStats(const QString &type);
    bool sendTopic(const QString &chan, const QString &topic);
    bool sendTopic(const QString &chan);
    bool sendVersion(const QString &to);
    bool sendWallops(const QString &msg);
    bool sendWho(const QString &mask);
    bool sendWhoami();
    bool sendWhois(const QString &params);
    bool sendWhowas(const QString &params);
    bool isUserIgnored(QString nick, QString user, QString host, QString on);
    bool isUserIgnored(QString nick, QString on);
    bool isAway(const QString &nick);
    QString getMyUserHost() const { return m_myUserHost; }
signals:
    void ircConnect(QString);
    void ircDisconnect(QString); //send at closeconnection w/o reconnection
    void ircReconnect(QString); //send at closeconnection with reconnection
    void ircReconnectAll(QString); //send at menu "Reconnect all"
    void ircError(QString);
    void ircNewChannel(QString);
    void ircQuery(QString);
    void ircPrivmsg(QString,QString,QString);
    void ircAction(QString,QString,QString);
    void ircCtcpReply(QString,QString);
    void ircCtcpRequest(QString,QString);
    void ircDccChat(QString,QString,int);
    void ircDccIn(QString,QString,QString,QString,QString);
    void ircDccToken(QString,QString,QString,int);
    void ircDccMyToken(QString,int,int);
    void ircDccResume(QString,QString,QString,QString);
    void ircDccPresume(int,QString);
    void ircDccAccept(QString,QString,QString,QString);
    void ircDccPaccept(int,QString);
    void ircJoin(QString,QString);
    void ircPart(QString,QString,QString);
    void ircQuit(QString,QString);
    void ircUnknown(QString,QString);
    void ircTopic(QString,QString,QString);
    void ircNick(QString,QString);
    void ircMode(QString,QString);
    void ircUmode(QString,QString,QString,QString);
    void ircChmode(QString,QString);
    void ircServerReply(QString); //command 001,002,....
    void ircServerError(QString); //command 400~599
    void irc301(QString,QString); //RPL_AWAY
    void irc305(QString); //RPL_UNAWAY
    void irc306(QString); //RPL_NOWAWAY
    void irc323(); //RPL_LISTEND
    void irc331(QString,QString); //RPL_NOTOPIC
    void irc332(QString,QString); //RPL_TOPIC
    void irc333(QString,QString); //RPL_TOPICSETBY
    void irc353(QString,QString,bool); //RPL_NAMREPLY
    void irc366(QString,QString,bool); //RPL_ENDOFNAMES
    void irc372(QString); //RPL_MOTD
    void irc401(QString); //ERR_NOSUCHNICK
    void ircKick(QString,QString,QString,QString);
    void ircNotice(QString,QString);
    void ircChnotice(QString,QString,QString);
    void ircInvite(QString,QString);
    void ircAway(QString);
    void ircEndMotd();
    void ircRaw(QString);
    void ircNickInfo(NickInfo);
    void ircBoatsInvite(QString);
    void ircBoatsAccept(QString);
    void ircBoatsDecline(QString);
    void ircBoatsNewGame(QString);
    void ircBoatsReady(QString);
    void ircBoats(QString,QString,QString);
    void ircBoatsLeft(QString);
    void ircBoatsDestroyed(QString);
    void ircBoatsWin(QString);
private:
    bool m_connected, m_useSsl, m_connecting, m_endmotd;
    bool m_ignoreUserHost, m_storeChannelList;
    int m_serverPort, m_attempts;
    int m_nickLen, m_topicLen, m_kickLen, m_awayLen;
    QString m_serverName, m_networkName, m_serverPassword, m_nickName;
    QString m_realName, m_userName, m_startChannels, m_startCommands;
    QString m_receiveRest;
    QString m_chanTypes; //channel prefixes
    QString m_myUserHost; //usefull for dcc
    QString m_ignoreWho;
    char m_adminPrefix, m_ownerPrefix, m_opPrefix, m_voicePrefix, m_halfopPrefix; //prefix for nick modes
    QList<QString> m_ignoreWhoQueue, m_namesQueue;
    QList<NickInfo> m_nicks;
    qlonglong m_ping;
    dxSocket *m_socket;
    QTimer *m_timerReconnect;
    QTimer *m_timerTryagain;
    QTimer *m_timerPing;
    QStringList m_channels, m_listChannels;

    void readData();
    bool sendLine(const QString &line);
    void sendCommands();
    bool sendCommand(const QString &commandtext);
    void parseLine(const QString &line);
    void numeric(const int&, const QString&);
    void privmsg(const QString&, const QString&);
    void ctcp(const QString&, const QString&);
    void join(const QString&, const QString&);
    void quitirc(const QString&, const QString&);
    void part(const QString&, const QString&);
    void ping(const QString&);
    void pong(const QString&, const QString&);
    void notice(const QString&, const QString&);
    void nick(const QString&, const QString&);
    void topic(const QString&, const QString&);
    void invite(const QString&, const QString&);
    void kick(const QString&, const QString&);
    void mode(const QString&, const QString&);
    void error(const QString&);
    void unknown(const QString&, const QString&);
    void parseRplsupport(QString);
    void makeStartChannels();
    void clearChannelsCommands(bool);
    void resetReconnect();
    void sendPingTimeout();
    bool hasReconnectAttempt();
    
private slots:
    void slotConnected();
    void slotDisconnected();
    void slotError(QString);
    void slotCanRead();
    void slotReconnect();
    void slotTryagain();
    void slotPing();
};

#endif // IRCENGINE_H
