/*
 *      fttabitem.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "fttabitem.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "preferences.h"
#include "dxirc.h"

static int percent(DccFile ft)
{
    if(ft.finished) return 100;
    quint64 position;
    if(ft.type == DCC_IN || ft.type == DCC_PIN)
        position = ft.currentPosition;
    else
        position = ft.finishedPosition;
    if(ft.size) return (int)100*position/ft.size;
    else return 0;
}

static QString statusForFT(DccFile ft)
{
    if(ft.canceled) return _("Canceled");
    if(!ft.started) return _("Waiting");
    if(ft.finished) return _("Finished");
    if(ft.type==DCC_IN || ft.type==DCC_PIN) return _("Downloading %1 of %2").arg(dxutils::getFileSize(ft.currentPosition)).arg(dxutils::getFileSize(ft.size));
    else return _("Uploading %1 of %2").arg(dxutils::getFileSize(ft.finishedPosition)).arg(dxutils::getFileSize(ft.size));
}

class FTViewDelegate : public QItemDelegate
{
public:
    inline FTViewDelegate(FtTabItem *item) : QItemDelegate(item) {}
    inline void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
    {
        Q_UNUSED(painter)
        Q_UNUSED(option)
        Q_UNUSED(index)
        if(index.column()!=1)
        {
            QItemDelegate::paint(painter, option, index);
            return;
        }
        QStyleOptionProgressBar progressBarOption;
        progressBarOption.state = QStyle::State_Enabled;
        progressBarOption.direction = QApplication::layoutDirection();
        progressBarOption.rect = option.rect;
        progressBarOption.fontMetrics = QApplication::fontMetrics();
        progressBarOption.minimum = 0;
        progressBarOption.maximum = 100;
        progressBarOption.textAlignment = Qt::AlignCenter;
        progressBarOption.textVisible = true;
        progressBarOption.progress = percent(qobject_cast<FtTabItem*>(parent())->ftForRow(index.row()));
        progressBarOption.text = statusForFT(qobject_cast<FtTabItem*>(parent())->ftForRow(index.row()));
        QApplication::style()->drawControl(QStyle::CE_ProgressBar, &progressBarOption, painter);
    }
};

FtTabItem::FtTabItem(QWidget *parent) :
    QWidget(parent)
{
    QStringList headers;
    headers << _("File") << _("Status") << _("Remaining time");
    m_fileView = new QTreeWidget(this);
    m_fileView->setItemDelegate(new FTViewDelegate(this));
    m_fileView->setHeaderLabels(headers);
    m_fileView->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_fileView->setAlternatingRowColors(true);
    m_fileView->setRootIsDecorated(false);
    m_fileView->setContextMenuPolicy(Qt::CustomContextMenu);
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(m_fileView);
    setLayout(mainLayout);
    QFontMetrics fm = fontMetrics();
    QHeaderView *header = m_fileView->header();
    header->resizeSection(0, fm.width("  Downloading 1024 MB from 1024 MB for 666KB/s .."));
    header->resizeSection(1, fm.width("  Downloading 1024 MB from 1024 MB for 666KB/s .."));
    header->resizeSection(2, fm.width(headers.at(2) + "  "));

    m_details = new QGroupBox(_("Details"));
    QGridLayout *groupLayout = new QGridLayout;
    groupLayout->addWidget(new QLabel(_("Path:")), 0, 0);
    m_path = new QLabel;
    groupLayout->addWidget(m_path, 0, 1);
    groupLayout->addWidget(new QLabel(_("IP address:")), 1, 0);
    m_ip = new QLabel;
    groupLayout->addWidget(m_ip, 1, 1);
    groupLayout->addWidget(new QLabel(_("Port:")), 2, 0);
    m_port = new QLabel;
    groupLayout->addWidget(m_port, 2, 1);
    groupLayout->addWidget(new QLabel(_("Nick:")), 3, 0);
    m_nick = new QLabel;
    groupLayout->addWidget(m_nick, 3, 1);
    m_details->setLayout(groupLayout);
    mainLayout->addWidget(m_details);
    m_details->hide();

    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Close);
    connect(buttons, SIGNAL(rejected()), this, SLOT(slotOk()));
    m_canceled = new QPushButton(_("Clear &canceled"));
    m_canceled->setEnabled(false);
    buttons->addButton(m_canceled, QDialogButtonBox::ActionRole);
    connect(m_canceled, SIGNAL(clicked()), this, SLOT(slotClearCanceled()));
    m_finished = new QPushButton(_("Clear &finished"));
    m_finished->setEnabled(false);
    buttons->addButton(m_finished, QDialogButtonBox::ActionRole);
    connect(m_finished, SIGNAL(clicked()), this, SLOT(slotClearFinished()));
    mainLayout->addWidget(buttons);
    
    m_position = new QTimer(this);

    connect(m_fileView, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this, SLOT(slotItemChanged(QTreeWidgetItem*)));
    connect(m_fileView, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(slotItemPopup(const QPoint &)));
    connect(m_position, SIGNAL(timeout()), this, SLOT(slotPosition()));
    
    m_position->start(1000);
}

DccFile FtTabItem::ftForRow(int row)
{
    return dxirc::instance()->getDccFiles().at(row);
}

int FtTabItem::rowForFT(DccFile ft) const
{
    int row = 0;
    foreach(DccFile item, dxirc::instance()->getDccFiles())
    {
        if(item==ft) return row;
        ++row;
    }
    return -1;
}

void FtTabItem::slotOk()
{
    emit closeFtTabItem();
}

void FtTabItem::slotClearCanceled()
{
    for(int i=dxirc::instance()->getDccFiles().size()-1; i>-1; i--)
    {
        if(dxirc::instance()->getDccFiles().at(i).canceled)
        {
            dxirc::instance()->getDccFiles().removeAt(i);
            m_fileView->takeTopLevelItem(i);
        }
    }
    m_canceled->setEnabled(false);
    if(!dxirc::instance()->getDccFiles().size())
        m_details->hide();
}

void FtTabItem::slotClearFinished()
{
    for(int i=dxirc::instance()->getDccFiles().size()-1; i>-1; i--)
    {
        if(dxirc::instance()->getDccFiles()[i].finished)
        {
            dxirc::instance()->getDccFiles().removeAt(i);
            m_fileView->takeTopLevelItem(i);
        }
    }
    m_finished->setEnabled(false);
    if(!dxirc::instance()->getDccFiles().size())
        m_details->hide();
}

void FtTabItem::slotItemChanged(QTreeWidgetItem* current)
{
    int row = m_fileView->indexOfTopLevelItem(current);
    if(row>=0 && row<dxirc::instance()->getDccFiles().size())
    {
        DccFile transfer = ftForRow(row);
        m_path->setText(transfer.path);
        m_ip->setText(transfer.ip);
        m_port->setText(QString::number(transfer.port));
        m_nick->setText(transfer.nick);
    }
}

void FtTabItem::slotItemPopup(const QPoint &pos)
{
    QTreeWidgetItem *item = 0;
    item = m_fileView->itemAt(pos);
    if(item)
    {
        int row = m_fileView->indexOfTopLevelItem(item);
        DccFile transfer = ftForRow(row);
        if(!transfer.canceled && !transfer.finished)
        {
            QMenu popup;
            QAction *action = new QAction(_("Cancel"), m_fileView);
            connect(action, SIGNAL(triggered()), this, SLOT(slotItemCancel()));
            popup.addAction(action);
            popup.exec(m_fileView->viewport()->mapToGlobal(pos));
        }
    }
}

void FtTabItem::slotItemCancel()
{
    emit cancelFt(m_fileView->indexOfTopLevelItem(m_fileView->currentItem()));
}

void FtTabItem::slotPosition()
{
    bool canceled = false;
    bool finished = false;
    foreach(DccFile item, dxirc::instance()->getDccFiles())
    {
        int row = rowForFT(item);
        QTreeWidgetItem *titem = m_fileView->topLevelItem(row);
        if(titem)
        {
            titem->setText(1, statusForFT(item));
            if(item.canceled || item.finished)
            {
                titem->setText(2, "--:--");
                if(item.canceled) canceled = true;
                else finished = true;
            }
            else
            {
                quint64 bytes;
                if(item.type == DCC_IN || item.type == DCC_PIN)
                    bytes = item.currentPosition;
                else
                    bytes = item.finishedPosition;
                titem->setText(2, dxutils::getRemaining(item.size-bytes, item.speed));
            }
        }
        else
        {
            titem = new QTreeWidgetItem(m_fileView);
            titem->setText(0, QFileInfo(item.path).fileName());
            titem->setText(1, _("Waiting"));
            titem->setText(2, dxutils::getRemaining(0,0));
            titem->setFlags(titem->flags() & ~Qt::ItemIsEditable);
            titem->setTextAlignment(1, Qt::AlignHCenter);
            m_fileView->setCurrentItem(titem);
            m_details->show();
        }
    }
    if(canceled) m_canceled->setEnabled(true);
    else m_canceled->setEnabled(false);
    if(finished) m_finished->setEnabled(true);
    else m_finished->setEnabled(false);
    int row = m_fileView->indexOfTopLevelItem(m_fileView->currentItem());
    if(row>=0 && row<dxirc::instance()->getDccFiles().size())
    {
        DccFile transfer = ftForRow(row);
        m_path->setText(transfer.path);
        m_ip->setText(transfer.ip);
        m_port->setText(QString::number(transfer.port));
        m_nick->setText(transfer.nick);
    }
}
